# MechaRaptor controller

The **MechaRaptor** is a tandem flapping wing system built for my Ph.D. at the
University of Liège. The goal of this setup is to conduct experimental analysis
of the aerodynamics of tandem flapping wings and better understand the wake
dynamics and how it impacts the lift and drag of the aft wing.

The system is actuated by two sets of brushless DC motors (BLDC), each
controlled with an Engine Speed Controller (ESC). The ESCs are themselves
controlled using an Arduino Uno. The Arduino acts as the brain of the whole
system and is the main interface for piloting the setup and logging important
metrics.

This repository contains the Arduino code and the wiring schematics for the
electrical components of the setup.

## Controller features

- Manual (potentiometer) or automatic (serial) control of the BLDCs.
- Both BLDCs run at the same speed (same flapping frequency on each module).
- Phase difference manually set before the experiment by changing the initial
  position of the wings.

_Note: A maximum rotation speed is imposed to the BLDC by the ESC. This value
has been manually set for each ESC using the ESC's terminal (JETIBOX)._

## Usage

Before powering the Arduino, set the ON/OFF switch in the appropriate position:

- ON: Serial mode (control using a PC)
- OFF: Manual mode (control using the potentiometer)

Further switch changes during the Arduino runtime will not be accounted for.

### Manual mode

In this mode, the potentiometer is used to control the motors. When it is turned
at its maximum, the motors run at the maximum RPM allowed by the ESCs (typically
6000 RPM). Turn it all the way down to stop the motors.

### Serial mode

In this mode, the Arduino is controlled through the serial port of a computer.
If no computer is connected, nothing will happen.

When the serial connection is started, the console will prompt the user for the
desired frequency and will use that value to control the motors. The
potentiometer is completely bypassed.

**To turn off the motor, send 0 to the console at any moment.** This
will first decrease the RPM to a moderate speed and then turn off completely
the motor.

## Data logging and visualization

Besides controlling the system, the Arduino also logs the input data and some
measurements. The most important data are:

- ESC input setting
- Wing angles (for each wing)
- PSU current and tension

These logged data are transmitted through the serial connection and saved on the
computer that controls the Arduino for later analysis.

A [_json_ configuration file](controller/mechaRaptor.json) is provided in order
to visualize the output stream in real time with [serial-studio]. This also
allows the automatic logging and retention of all data in _csv_ spreadsheets.

## Bill of Materials

| Device            | Reference    | Usage    |
|------------------ | --------------- | --------------- |
| **BLDC**          | [Plettenberg][plettenberg] Advance 30 <br />_(Discontinued by manufacturer)_  | Motor |
| **ESC**           | [JETI SPIN 75 Pro Opto][jetispin] | Motor controller |
| **ESC Terminal**  | [JETIBOX][jetibox] | Program ESC, motor telemetry |
| **PSU**           | [Electroautomatik][elektro] PSI 8080 - 60 T (1500W) <br />_(Discontinued by manufacturer)_ | DC power supply |
| **Controller board**  | [Arduino uno (Rev 3)][arduino] | Full setup control, data logging|
| **Encoders**      | [Broadcom AEAT 6012][AEAT612] | Flapping angle measurement |
| **Potentiometer** | N/A | Manual control of RPM|
| **ON/OFF switch** | N/A | Switch between `Serial` and `Manual` control |
| **Resistors** | N/A | Voltage divider, noise reduction |
| **Current sensor** | [Allegro SEN0098][SEN0098] | PSU current logging |

## Schematics

To Do (January 2023)

[plettenberg]: <https://plettenbergmotors.com/>
[serial-studio]: <https://serial-studio.github.io/>
[jetispin]: <https://www.jetimodel.com/katalog/spin-75-pro-opto.htm>
[jetibox]: <https://www.jetimodel.com/katalog/jetibox.htm>
[arduino]: <https://store.arduino.cc/products/arduino-uno-rev3>
[elektro]: <https://elektroautomatik.de>
[AEAT612]: <https://docs.broadcom.com/doc/AV02-0188EN>
[SEN0098]: https://www.allegromicro.com/~/media/Files/Datasheets/ACS758-Datasheet.ashx
